from __future__ import absolute_import, division, print_function
import iotbx.pdb
import iotbx.mrcfile
import mmtbx.model
import mmtbx.real_space
# from cctbx import crystal
from scitbx.array_family import flex
# from cctbx.development import random_structure
# from cctbx import sgtbx
from cctbx import maptbx

apix = 2.0
# boxsize = 32

  # Read the model file
pdb_inp = iotbx.pdb.input(file_name = "4v7t.cif")
model = mmtbx.model.manager(model_input = pdb_inp)
model.setup_scattering_dictionaries(scattering_table="electron")
xrs = model.get_xray_structure()

# # Calculate structure factors at given resolution.
# f_calc = xrs.structure_factors(d_min = 2*apix).f_calc()

# # Write them down as MTZ file
# mtz_dataset = f_calc.as_mtz_dataset(column_root_label="F-calc")
# mtz_object = mtz_dataset.mtz_object()
# mtz_object.write(file_name = "f_calc.mtz")

# # Convert Fcalc into real map (just do FFT)
# fft_map = f_calc.fft_map(resolution_factor=1./4)
# fft_map.apply_sigma_scaling()
# map_data = fft_map.real_map_unpadded()

# print(f_calc.unit_cell())

# box = boxsize * apix
# symmetry = crystal.symmetry(
#     unit_cell=(box, box, box, 90, 90, 90),
#     space_group_symbol="P 1")

# Calculate exact map and write it down
crystal_gridding = maptbx.crystal_gridding(
  unit_cell        = xrs.unit_cell(),
  space_group_info = xrs.space_group_info(),
  symmetry_flags   = maptbx.use_space_group_symmetry,
  step             = apix)
m = mmtbx.real_space.sampled_model_density(
  xray_structure = xrs,
  n_real         = crystal_gridding.n_real())
#   n_real = [256, 256, 256])
map_data = m.data()

# Write real Fourier map into MRC file
# iotbx.mrcfile.write_ccp4_map(
#   file_name   = "fourier_map.mrc",
#   unit_cell   = f_calc.unit_cell(),
#   space_group = f_calc.crystal_symmetry().space_group(),
#   map_data    = map_data.as_double(),
#   labels      = flex.std_string(["Some text"]))

iotbx.mrcfile.write_ccp4_map(
  file_name   = "fourier_map.mrc",
  unit_cell   = xrs.unit_cell(),
  space_group = xrs.crystal_symmetry().space_group(),
  map_data    = map_data.as_double(),
  labels      = flex.std_string(["Some text"]))